/*
 *   Copyright 2008 Jaroslav Reznik <rezzabuh@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2 or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "atipsengine.h"

#include <QStringList>
#include <QTimer>

#include <KDebug>
#include <KProcess>

#include <Plasma/DataContainer>

using namespace Plasma;

AtiPSEngine::AtiPSEngine(QObject* parent, const QVariantList& args)
    : Plasma::DataEngine(parent), m_sources(0), m_powerstates(0), m_currentstate(0)
{
	Q_UNUSED(args)
	setMinimumUpdateInterval(333);

	// sources for plasmaengineexplorer
	m_sources << "PowerSave" << "Current";
}

AtiPSEngine::~AtiPSEngine()
{
}

bool AtiPSEngine::listPowerStates()
{
	// for now we are using QProcess due to bug in KProcess
	QProcess aticonfig;

	aticonfig.start("aticonfig --lsp");

	if (!aticonfig.waitForStarted())
		return false;

	aticonfig.waitForFinished();

	char buf[1024];
	int lines=0;

	// clear powerstates list
	m_powerstates.clear();

	// parse output
	while (aticonfig.readLine(buf, sizeof(buf)) > 0)
	{
		QString line = buf;

		lines++;
		
		// first line must contain "core/mem" string
		if (lines == 1)
		{
			if (!line.contains("core/mem"))
				return false;
		}
		else if (lines > 2)
		{
			if (line.startsWith("*"))
				m_currentstate = lines - 2;

			m_powerstates << line.section(": ", 1, 1);
		}
	}

	if (lines < 3)
		return false;
	else
		return true;

}

bool AtiPSEngine::sourceRequested(const QString &name)
{
	QStringList powerStates;
	bool support;

	// list powerstates
	support = listPowerStates();

	if (name == "PowerSave")
	{
		if (support)
			setData("PowerSave", "states", m_powerstates );

		setData("PowerSave", "supported", support);
	}
	else if (name == "Current")
		updateSource(name);

	return true;
}

bool AtiPSEngine::updateSource(const QString &source)
{
	Q_UNUSED(source)

	if (!listPowerStates())
	{
		setData("Current", "state", 0);
		setData("PowerSave", "supported", false);
	}
	else
	{
		setData("Current", "state", m_currentstate);
	}

	return true;
}

QStringList AtiPSEngine::sources() const 
{
    return m_sources;
}

#include "atipsengine.moc"
